package sample;

import javax.naming.InitialContext;

import com.ibm.bpe.api.BusinessFlowManager;
import com.ibm.bpe.api.BusinessFlowManagerHome;
import com.ibm.task.api.HumanTaskManager;
import com.ibm.task.api.HumanTaskManagerHome;

/**
 * Class providing methods to initialize access to the EJB APIs of Human Task
 * Manager, Business Flow Manager.
 */
public class BPCAccess {
	private static volatile HumanTaskManager _htm = null;
	private static volatile BusinessFlowManager _bfm = null;
	private static String _htmLock = "htmLock";
	private static String _bfmLock = "bfmLock";

	/**
	 * Method to initialize access to Human Task Manager EJB API.
	 * 
	 * @return the Human Task Manager access object.
	 */
	public static HumanTaskManager getHTM() {
		if (_htm == null) {
			// needs synchronization, due to concurrent plugin execution
			synchronized (_htmLock) {
				_htm = getNewHTM();
			}
		}
		return _htm;
	}

	/**
	 * @return a new Human Task Manager access object
	 */
	private static HumanTaskManager getNewHTM() {
		HumanTaskManager newHTM = null;

		try {
			InitialContext _initialHTMContext = new InitialContext();

			// use the fully qualified name
			Object result = _initialHTMContext
					.lookup("com/ibm/task/api/HumanTaskManagerHome");

			HumanTaskManagerHome htmHome = (HumanTaskManagerHome) javax.rmi.PortableRemoteObject
					.narrow(result, HumanTaskManagerHome.class);
			newHTM = htmHome.create();
		} catch (Exception e) {
			// tracing
			System.out
					.println("Could not initialize Human Task Manager access: "
							+ e.getMessage());
			throw new Error(e);
		}

		// return the access object, if one established, otherwise, null.
		return newHTM;
	}

	/**
	 * Method to initialize access to Business Flow Manager EJB API.
	 * 
	 * @return the Business Flow Manager access object.
	 */
	public static BusinessFlowManager getBFM() {
		if (_bfm == null) {
			// needs synchronization, due to concurrent plugin execution
			synchronized (_bfmLock) {
				_bfm = getNewBFM();
			}
		}
		return _bfm;
	}

	/**
	 * @return A new Business Flow Manager access object.
	 */
	private static BusinessFlowManager getNewBFM() {
		BusinessFlowManager newBFM = null;

		try {
			InitialContext _initialBFMContext = new InitialContext();

			// use the fully qualified name
			Object result = _initialBFMContext
					.lookup("com/ibm/bpe/api/BusinessFlowManagerHome");

			BusinessFlowManagerHome bfmHome = (BusinessFlowManagerHome) javax.rmi.PortableRemoteObject
					.narrow(result, BusinessFlowManagerHome.class);
			newBFM = bfmHome.create();
		} catch (Exception e) {
			// tracing
			System.out
					.println("Could not initialize Business Flow Manager access: "
							+ e.getMessage());
			throw new Error(e);
		}

		// return the access object, if one established, otherwise, null.
		return newBFM;
	}

}
