package sample;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Random;

import org.omg.stub.java.rmi._Remote_Stub;

/**
 * Class implementing the Random user selection policy.
 */
public class RandomSelectionControl {
	// Static randomizer object used for random user selections.
	// Note: its nextInt() method has a thread-safe implementation in Java.
	private static Random _randomizer = new Random();

	/**
	 * Method selecting one user randomly from the set of users contained in the
	 * map.
	 * 
	 * @param userMap
	 *            The received map containing the set of user Ids as keys.
	 * @return The selected user Id.
	 */
	static String getUserSelection(Map userMap) {
		// determine next random selection index
		int index = _randomizer.nextInt() % userMap.size();

		// navigate to the computed user selection
		Iterator userIterator = userMap.keySet().iterator();
		for (int i = 0; i < index; i++) {
			userIterator.next();
		}
		String selectedUserId = (String) userIterator.next();

		// return the randomly selected user Id
		return selectedUserId;
	}

	// Local test method, used for testing purposes, only.
	// Not invoked during regular plugin execution.
	public static void main(String[] argv) {
		HashMap map = new HashMap();
		map.put("user1", "user1");
		map.put("user2", "user2");
		map.put("user3", "user3");
		map.put("user4", "user4");
		map.put("user5", "user5");
		map.put("user6", "user6");
		map.put("user7", "user7");

		for (int i = 0; i < 10; i++) {
			System.out.println("randmom user: " + i + " is: "
					+ getUserSelection(map));
		}
	}

}
